<?php namespace HM\BackUpWordPress;

/**
 * Class DropboxAddon
 * @package HM\BackUpWordPress
 */
class DropboxAddon extends Addon {

	public function __construct( $plugin_version, $min_bwp_version, $service_class, $edd_download_file_name, $plugin_name, $plugin_settings_key, $plugin_settings_defaults ) {

		parent::__construct( $plugin_version, $min_bwp_version, $service_class, $edd_download_file_name, $plugin_name, $plugin_settings_key, $plugin_settings_defaults );

			add_action( 'admin_post_hmbkpp_dropbox_auth_step_1', array( $this, 'auth_step_1' ) );

			add_action( 'admin_post_hmbkpp_dropbox_auth_step_2', array( $this, 'auth_step_2' ) );

			add_action( 'admin_post_hmbkpp_dropbox_clear_auth', array( $this, 'unauth' ) );

	}

	/**
	 * Clear authentication with Dropbox.
	 */
	public function unauth() {

		check_admin_referer( 'dropbox-clear-auth', 'dropbox-clear-auth-nonce' );

		if ( ! current_user_can( 'manage_options' ) ) {
			return;
		}

		$settings = $this->fetch_settings();

		$settings['access_token'] = '';
		$settings['request_token'] = '';
		$settings['account_info'] = '';

		update_site_option( 'hmbkpp_dx_settings', $settings );

		wp_safe_redirect( wp_get_referer() ); die;

	}

	public function auth_step_1() {

		check_admin_referer( 'dropbox-auth-step1', 'dropbox-auth-step1-nonce' );

		if ( empty( $_GET['action'] ) || 'hmbkpp_dropbox_auth_step_1' !== $_GET['action'] ) {
			return;
		}

		$settings = $this->fetch_settings();

		// abort if schedule ID is missing
		if ( empty( $_GET['hmbkp_schedule_id'] ) ) {
			wp_die( 'error' );
		}

		$schedule_id = sanitize_text_field( $_GET['hmbkp_schedule_id'] );

		// this call will set the request token and redirect to dropbox auth page
		$this->bootstrap_service( $schedule_id );

		die;

	}

	public function auth_step_2() {

		check_admin_referer( 'dropbox-auth-step2', 'dropbox-auth-step2-nonce' );

		if ( empty( $_GET['action'] ) || 'hmbkpp_dropbox_auth_step_2' !== $_GET['action'] ) {
			return;
		}

		// abort if schedule ID is missing
		if ( empty( $_GET['hmbkp_schedule_id'] ) ) {
			return;
		}

		$settings = $this->fetch_settings();

		if ( empty( $settings['access_token'] ) ) {
			$schedule_id = sanitize_text_field( $_GET['hmbkp_schedule_id'] );

			// this call will set the access token after coming back from the dropbox auth page
			$this->bootstrap_service( $schedule_id );
		}

		//$dx_settings_tab = add_query_arg( array( 'action' => 'hmbkp_edit_schedule', 'hmbkp_panel' => 'hmbkp_edit_schedule_settings_dropbox', 'hmbkp_schedule_id' => $schedule_id ), HMBKP_ADMIN_URL );

		wp_redirect( HMBKP_ADMIN_URL, 303 );
		exit;

	}

	/**
	 * Determines the Schedules Dropbox service and initiates bootstrap
	 *
	 * @param $schedule_id
	 */
	public function bootstrap_service( $schedule_id ) {

		$schedules = Schedules::get_instance();

		$schedule = $schedules->get_schedule( $schedule_id );

		foreach ( Services::get_services( $schedule ) as $service ) {

			if ( 'Dropbox' === $service->name ) {
				/** get request token, this will cause a redirect to a dropbox page to authorize
				 * the app to access the users dropbox. on return, we can hook into admin_init
				 * and check the response via a $_GET var
				 */
				$service->bootstrap( $schedule_id );

			}
		}

	}

	public function fetch_settings() {

		return get_site_option( $this->plugin_settings_key, $this->plugin_settings_defaults );
	}
}
